/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is include which follows the "include" declaration
** in the input file. */
#include <stdio.h>
#line 1 "parser.lem"

/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

	 
	#include <math.h>
	#include "log.h"
	#include "tokenizer.h"
	#include "parser.h"
	
	#define Ln(x)	log(fabs(x))
	#define Div(x,y,m) \
		((fabs(y) > (m))?((x)/(y)):(((y) > 0)?(x)/(m):(x)/(-m)))
	
	typedef struct {
		double f;	/* value of function */
		double d;	/* value of derivative */
	} derivData;
#line 46 "parser.c"
#include "parser.h"
/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
#if INTERFACE
#define TOKEN_PLUS                            1
#define TOKEN_MINUS                           2
#define TOKEN_DIVIDE                          3
#define TOKEN_TIMES                           4
#define TOKEN_POWER                           5
#define TOKEN_NOT                             6
#define TOKEN_ABS                             7
#define TOKEN_ACOSH                           8
#define TOKEN_ACOS                            9
#define TOKEN_ASINH                          10
#define TOKEN_ASIN                           11
#define TOKEN_ATANH                          12
#define TOKEN_ATAN                           13
#define TOKEN_COSH                           14
#define TOKEN_COS                            15
#define TOKEN_EXP                            16
#define TOKEN_LN                             17
#define TOKEN_LOG                            18
#define TOKEN_SINH                           19
#define TOKEN_SIN                            20
#define TOKEN_SQRT                           21
#define TOKEN_TAN                            22
#define TOKEN_URAMP                          23
#define TOKEN_U                              24
#define TOKEN_LPAREN                         25
#define TOKEN_RPAREN                         26
#define TOKEN_DIFF                           27
#define TOKEN_VARIABLE                       28
#define TOKEN_ZERO_VARIABLE                  29
#define TOKEN_CONSTANT                       30
#define TOKEN_DIFF_VARIABLE                  31
#endif
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into yypParser
**    ParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 39
#define YYACTIONTYPE unsigned char
#if INTERFACE
#define ParseTOKENTYPE  token_* 
#endif
typedef union {
  ParseTOKENTYPE yy0;
  derivData yy17;
  double yy60;
  int yy77;
} YYMINORTYPE;
#define YYSTACKDEPTH 100
#if INTERFACE
#define ParseARG_SDECL  double *solution ;
#define ParseARG_PDECL , double *solution 
#define ParseARG_FETCH  double *solution  = yypParser->solution 
#define ParseARG_STORE yypParser->solution  = solution 
#endif
#define YYNSTATE 150
#define YYNRULE 60
#define YYERRORSYMBOL 32
#define YYERRSYMDT yy77
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* Next are that tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static const YYACTIONTYPE yy_action[] = {
 /*     0 */    44,   64,    1,    3,   48,   17,   22,   25,   28,   31,
 /*    10 */    34,   37,   40,   43,   46,   49,   51,    4,    7,   10,
 /*    20 */    13,   16,   19,   32,    6,   18,  121,  113,  102,   45,
 /*    30 */    42,   39,   36,   33,   30,   27,   24,   21,   14,    8,
 /*    40 */     2,   47,   38,   29,   26,   23,   35,   12,   80,   44,
 /*    50 */   131,   96,  132,  130,   17,   22,   25,   28,   31,   34,
 /*    60 */    37,   40,   43,   46,   49,   51,    4,    7,   10,   13,
 /*    70 */    16,   19,   32,   65,  138,  121,  113,  102,   41,   20,
 /*    80 */     5,   50,   11,   15,    9,    1,    3,   48,   15,    9,
 /*    90 */     1,    3,   48,   15,    9,    1,    3,   48,   15,    9,
 /*   100 */     1,    3,   48,  106,   69,   91,  127,  110,  105,  211,
 /*   110 */     5,   50,   11,  108,   90,   97,   95,   93,  114,   67,
 /*   120 */   100,   83,   54,  117,   72,   15,    9,    1,    3,   48,
 /*   130 */    15,    9,    1,    3,   48,   15,    9,    1,    3,   48,
 /*   140 */    15,    9,    1,    3,   48,   15,    9,    1,    3,   48,
 /*   150 */   120,   99,   68,   82,   81,  122,   66,   48,  118,   71,
 /*   160 */   123,   70,   11,   98,  100,  134,   53,   94,   77,   59,
 /*   170 */   149,   58,   15,    9,    1,    3,   48,   15,    9,    1,
 /*   180 */     3,   48,   15,    9,    1,    3,   48,   15,    9,    1,
 /*   190 */     3,   48,   15,    9,    1,    3,   48,  107,   89,   60,
 /*   200 */    78,  100,  119,   76,   86,   57,   85,  125,  100,   52,
 /*   210 */    75,   87,  116,   61,   79,   56,   92,  111,   62,   41,
 /*   220 */    20,    5,   50,   11,   41,   20,    5,   50,   11,   41,
 /*   230 */    20,    5,   50,   11,   41,   20,    5,   50,   11,   15,
 /*   240 */     9,    1,    3,   48,  144,  124,   74,   88,   63,  115,
 /*   250 */    73,  100,   55,   84,  112,  100,  142,  100,  100,  109,
 /*   260 */   100,  100,  100,  100,  103,  100,   41,   20,    5,   50,
 /*   270 */    11,   41,   20,    5,   50,   11,   41,   20,    5,   50,
 /*   280 */    11,   41,   20,    5,   50,   11,   41,   20,    5,   50,
 /*   290 */    11,  104,  100,  100,  100,  100,  101,  100,  100,  100,
 /*   300 */   100,  148,  100,  100,  100,  100,  146,  100,  100,  100,
 /*   310 */   100,  143,  100,   41,   20,    5,   50,   11,   41,   20,
 /*   320 */     5,   50,   11,   41,   20,    5,   50,   11,   41,   20,
 /*   330 */     5,   50,   11,   41,   20,    5,   50,   11,  141,  100,
 /*   340 */   100,  100,  100,  133,  100,  100,  100,  100,  139,  100,
 /*   350 */   100,  100,  100,  137,  100,  100,  100,  100,  136,  100,
 /*   360 */    41,   20,    5,   50,   11,   41,   20,    5,   50,   11,
 /*   370 */    41,   20,    5,   50,   11,   41,   20,    5,   50,   11,
 /*   380 */    15,    9,    1,    3,   48,  140,  100,  100,  100,  100,
 /*   390 */   129,  100,  100,  100,  100,  128,  100,  100,  100,  100,
 /*   400 */   126,  100,  100,  100,  100,  135,  100,   15,    9,    1,
 /*   410 */     3,   48,   15,    9,    1,    3,   48,   15,    9,    1,
 /*   420 */     3,   48,   15,    9,    1,    3,   48,   41,   20,    5,
 /*   430 */    50,   11,  145,  100,  100,  100,  100,  147,  100,  100,
 /*   440 */   100,  100,  100,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */     2,   33,    3,    4,    5,    7,    8,    9,   10,   11,
 /*    10 */    12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
 /*    20 */    22,   23,   24,   25,    2,   27,   28,   29,   30,    7,
 /*    30 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*    40 */    18,   19,   20,   21,   22,   23,   24,   25,   36,    2,
 /*    50 */    28,   33,   30,   31,    7,    8,    9,   10,   11,   12,
 /*    60 */    13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
 /*    70 */    23,   24,   25,   33,   36,   28,   29,   30,    1,    2,
 /*    80 */     3,    4,    5,    1,    2,    3,    4,    5,    1,    2,
 /*    90 */     3,    4,    5,    1,    2,    3,    4,    5,    1,    2,
 /*   100 */     3,    4,    5,   26,   33,   33,   34,   35,   26,   37,
 /*   110 */     3,    4,    5,   26,   36,   36,   36,   33,   26,   33,
 /*   120 */    38,   36,   33,   26,   36,    1,    2,    3,    4,    5,
 /*   130 */     1,    2,    3,    4,    5,    1,    2,    3,    4,    5,
 /*   140 */     1,    2,    3,    4,    5,    1,    2,    3,    4,    5,
 /*   150 */    26,   36,   33,   36,   36,   26,   33,    5,   33,   33,
 /*   160 */    26,   33,    5,   33,   38,   26,   33,   36,   36,   33,
 /*   170 */    26,   33,    1,    2,    3,    4,    5,    1,    2,    3,
 /*   180 */     4,    5,    1,    2,    3,    4,    5,    1,    2,    3,
 /*   190 */     4,    5,    1,    2,    3,    4,    5,   26,   36,   33,
 /*   200 */    36,   38,   26,   36,   36,   33,   36,   26,   38,   36,
 /*   210 */    36,   36,   26,   33,   36,   33,   33,   26,   33,    1,
 /*   220 */     2,    3,    4,    5,    1,    2,    3,    4,    5,    1,
 /*   230 */     2,    3,    4,    5,    1,    2,    3,    4,    5,    1,
 /*   240 */     2,    3,    4,    5,   26,   33,   36,   36,   33,   26,
 /*   250 */    36,   38,   33,   36,   26,   38,   36,   38,   38,   26,
 /*   260 */    38,   38,   38,   38,   26,   38,    1,    2,    3,    4,
 /*   270 */     5,    1,    2,    3,    4,    5,    1,    2,    3,    4,
 /*   280 */     5,    1,    2,    3,    4,    5,    1,    2,    3,    4,
 /*   290 */     5,   26,   38,   38,   38,   38,   26,   38,   38,   38,
 /*   300 */    38,   26,   38,   38,   38,   38,   26,   38,   38,   38,
 /*   310 */    38,   26,   38,    1,    2,    3,    4,    5,    1,    2,
 /*   320 */     3,    4,    5,    1,    2,    3,    4,    5,    1,    2,
 /*   330 */     3,    4,    5,    1,    2,    3,    4,    5,   26,   38,
 /*   340 */    38,   38,   38,   26,   38,   38,   38,   38,   26,   38,
 /*   350 */    38,   38,   38,   26,   38,   38,   38,   38,   26,   38,
 /*   360 */     1,    2,    3,    4,    5,    1,    2,    3,    4,    5,
 /*   370 */     1,    2,    3,    4,    5,    1,    2,    3,    4,    5,
 /*   380 */     1,    2,    3,    4,    5,   26,   38,   38,   38,   38,
 /*   390 */    26,   38,   38,   38,   38,   26,   38,   38,   38,   38,
 /*   400 */    26,   38,   38,   38,   38,   26,   38,    1,    2,    3,
 /*   410 */     4,    5,    1,    2,    3,    4,    5,    1,    2,    3,
 /*   420 */     4,    5,    1,    2,    3,    4,    5,    1,    2,    3,
 /*   430 */     4,    5,   26,   38,   38,   38,   38,   26,   38,   38,
 /*   440 */    38,   38,   26,
};
#define YY_SHIFT_USE_DFLT (-3)
#define YY_SHIFT_MAX 99
static const short yy_shift_ofst[] = {
 /*     0 */    -2,   22,   22,   22,   47,   47,   22,   47,   22,   22,
 /*    10 */    47,   47,   22,   47,   22,   22,   47,   47,   22,   47,
 /*    20 */    47,   22,   47,   22,   22,   47,   22,   22,   47,   22,
 /*    30 */    22,   47,   47,   22,   47,   22,   22,   47,   22,   22,
 /*    40 */    47,   47,   22,   47,   47,   22,   47,   22,   22,   47,
 /*    50 */    47,   47,  191,  218,  223,  228,  233,   77,  265,  270,
 /*    60 */   275,  280,  285,  312,  317,  322,  327,  332,  359,  364,
 /*    70 */   369,  374,  379,  406,  411,  416,  238,   82,   87,   92,
 /*    80 */    97,  124,  129,  134,  139,  144,  171,  176,  181,  186,
 /*    90 */   421,  426,  107,  107,   -1,   -1,  157,  152,  157,  152,
};
#define YY_REDUCE_USE_DFLT (-33)
#define YY_REDUCE_MAX 51
static const short yy_reduce_ofst[] = {
 /*     0 */    72,   79,   85,  115,  123,  130,   38,   86,  117,  131,
 /*    10 */   119,  125,   88,   71,  118,   80,  128,   89,   78,  126,
 /*    20 */    84,   12,  219,  211,  178,  182,  175,  173,  172,  168,
 /*    30 */   164,  138,  133,  132,  136,  162,  167,  166,  170,  174,
 /*    40 */   180,  183,  210,  185,  212,  214,  215,  217,  220,  -32,
 /*    50 */    18,   40,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    10 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    20 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    30 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    40 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    50 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    60 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    70 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    80 */   210,  210,  210,  210,  210,  210,  210,  210,  210,  210,
 /*    90 */   181,  151,  159,  158,  188,  187,  160,  190,  161,  189,
 /*   100 */   194,  168,  155,  195,  167,  196,  166,  206,  197,  165,
 /*   110 */   150,  198,  164,  154,  199,  163,  209,  200,  162,  207,
 /*   120 */   201,  153,  202,  203,  157,  208,  180,  152,  179,  178,
 /*   130 */   182,  183,  184,  173,  204,  185,  176,  175,  186,  174,
 /*   140 */   177,  172,  191,  171,  156,  192,  170,  193,  169,  205,
};
#define YY_SZ_ACTTAB (int)(sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammer, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  int stateno;       /* The state-number */
  int major;         /* The major token value.  This is the code
                     ** number for the token at this stack level */
  YYMINORTYPE minor; /* The user-supplied minor token value.  This
                     ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
  int yyerrcnt;                 /* Shifts left before out of the error */
  ParseARG_SDECL                /* A place to hold %extra_argument */
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "PLUS",          "MINUS",         "DIVIDE",      
  "TIMES",         "POWER",         "NOT",           "ABS",         
  "ACOSH",         "ACOS",          "ASINH",         "ASIN",        
  "ATANH",         "ATAN",          "COSH",          "COS",         
  "EXP",           "LN",            "LOG",           "SINH",        
  "SIN",           "SQRT",          "TAN",           "URAMP",       
  "U",             "LPAREN",        "RPAREN",        "DIFF",        
  "VARIABLE",      "ZERO_VARIABLE",  "CONSTANT",      "DIFF_VARIABLE",
  "error",         "expr",          "diff_expr",     "ans",         
  "diff",          "program",     
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "program ::= ans",
 /*   1 */ "ans ::= expr",
 /*   2 */ "ans ::= diff_expr",
 /*   3 */ "expr ::= VARIABLE",
 /*   4 */ "expr ::= ZERO_VARIABLE",
 /*   5 */ "expr ::= CONSTANT",
 /*   6 */ "expr ::= LPAREN expr RPAREN",
 /*   7 */ "expr ::= MINUS expr",
 /*   8 */ "expr ::= expr MINUS expr",
 /*   9 */ "expr ::= expr PLUS expr",
 /*  10 */ "expr ::= expr TIMES expr",
 /*  11 */ "expr ::= expr DIVIDE expr",
 /*  12 */ "expr ::= expr POWER expr",
 /*  13 */ "expr ::= ABS expr RPAREN",
 /*  14 */ "expr ::= ACOSH expr RPAREN",
 /*  15 */ "expr ::= ACOS expr RPAREN",
 /*  16 */ "expr ::= ASINH expr RPAREN",
 /*  17 */ "expr ::= ASIN expr RPAREN",
 /*  18 */ "expr ::= ATANH expr RPAREN",
 /*  19 */ "expr ::= ATAN expr RPAREN",
 /*  20 */ "expr ::= COSH expr RPAREN",
 /*  21 */ "expr ::= COS expr RPAREN",
 /*  22 */ "expr ::= EXP expr RPAREN",
 /*  23 */ "expr ::= LN expr RPAREN",
 /*  24 */ "expr ::= LOG expr RPAREN",
 /*  25 */ "expr ::= SINH expr RPAREN",
 /*  26 */ "expr ::= SIN expr RPAREN",
 /*  27 */ "expr ::= SQRT expr RPAREN",
 /*  28 */ "expr ::= TAN expr RPAREN",
 /*  29 */ "expr ::= URAMP expr RPAREN",
 /*  30 */ "expr ::= U expr RPAREN",
 /*  31 */ "diff_expr ::= DIFF diff",
 /*  32 */ "diff ::= DIFF_VARIABLE",
 /*  33 */ "diff ::= VARIABLE",
 /*  34 */ "diff ::= CONSTANT",
 /*  35 */ "diff ::= LPAREN diff RPAREN",
 /*  36 */ "diff ::= MINUS diff",
 /*  37 */ "diff ::= diff PLUS diff",
 /*  38 */ "diff ::= diff MINUS diff",
 /*  39 */ "diff ::= diff TIMES diff",
 /*  40 */ "diff ::= diff DIVIDE diff",
 /*  41 */ "diff ::= diff POWER diff",
 /*  42 */ "diff ::= ABS diff RPAREN",
 /*  43 */ "diff ::= ACOSH diff RPAREN",
 /*  44 */ "diff ::= ACOS diff RPAREN",
 /*  45 */ "diff ::= ASINH diff RPAREN",
 /*  46 */ "diff ::= ASIN diff RPAREN",
 /*  47 */ "diff ::= ATANH diff RPAREN",
 /*  48 */ "diff ::= ATAN diff RPAREN",
 /*  49 */ "diff ::= COSH diff RPAREN",
 /*  50 */ "diff ::= COS diff RPAREN",
 /*  51 */ "diff ::= EXP diff RPAREN",
 /*  52 */ "diff ::= LN diff RPAREN",
 /*  53 */ "diff ::= LOG diff RPAREN",
 /*  54 */ "diff ::= SINH diff RPAREN",
 /*  55 */ "diff ::= SIN diff RPAREN",
 /*  56 */ "diff ::= SQRT diff RPAREN",
 /*  57 */ "diff ::= TAN diff RPAREN",
 /*  58 */ "diff ::= URAMP diff RPAREN",
 /*  59 */ "diff ::= U diff RPAREN",
};
#endif /* NDEBUG */

/*
** This function returns the symbolic name associated with a token
** value.
*/
const char *ParseTokenName(int tokenType){
#ifndef NDEBUG
  if( tokenType>0 && tokenType<(sizeof(yyTokenName)/sizeof(yyTokenName[0])) ){
    return yyTokenName[tokenType];
  }else{
    return "Unknown";
  }
#else
  return "";
#endif
}

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and ParseFree.
*/
void *ParseAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor( yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from ParseAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void ParseFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
  (*freeProc)((void*)pParser);
}

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  if( iLookAhead==YYNOCODE ){
    return YY_NO_ACTION;
  }
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      int iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( j>=0 && j<YY_SZ_ACTTAB && yy_lookahead[j]==YYWILDCARD ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  /* int stateno = pParser->yystack[pParser->yyidx].stateno; */
 
  if( stateno>YY_REDUCE_MAX ||
      (i = yy_reduce_ofst[stateno])==YY_REDUCE_USE_DFLT ){
    return yy_default[stateno];
  }
  if( iLookAhead==YYNOCODE ){
    return YY_NO_ACTION;
  }
  i += iLookAhead;
  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer ot the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
  if( yypParser->yyidx>=YYSTACKDEPTH ){
     ParseARG_FETCH;
     yypParser->yyidx--;
#ifndef NDEBUG
     if( yyTraceFILE ){
       fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
     }
#endif
     while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
     /* Here code is inserted which will execute if the parser
     ** stack every overflows */
#line 49 "parser.lem"
 Error("Parser stack overflow"); *solution = sqrtf(-1.f);
#line 721 "parser.c"
     ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
     return;
  }
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = yyNewState;
  yytos->major = yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 37, 1 },
  { 35, 1 },
  { 35, 1 },
  { 33, 1 },
  { 33, 1 },
  { 33, 1 },
  { 33, 3 },
  { 33, 2 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 33, 3 },
  { 34, 2 },
  { 36, 1 },
  { 36, 1 },
  { 36, 1 },
  { 36, 3 },
  { 36, 2 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
  { 36, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ParseARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

#ifndef NDEBUG
  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  */
  memset(&yygotominor, 0, sizeof(yygotominor));
#endif

  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 0:
#line 59 "parser.lem"
{ Debug("Solution: %e", yymsp[0].minor.yy60); *solution = yymsp[0].minor.yy60; }
#line 859 "parser.c"
        break;
      case 1:
      case 2:
#line 61 "parser.lem"
{ yygotominor.yy60 = yymsp[0].minor.yy60; }
#line 865 "parser.c"
        break;
      case 3:
#line 64 "parser.lem"
{ yygotominor.yy60 = **yymsp[0].minor.yy0->variable; }
#line 870 "parser.c"
        break;
      case 4:
#line 65 "parser.lem"
{ yygotominor.yy60 = 0.0; }
#line 875 "parser.c"
        break;
      case 5:
#line 66 "parser.lem"
{ yygotominor.yy60 = yymsp[0].minor.yy0->constant; }
#line 880 "parser.c"
        break;
      case 6:
#line 67 "parser.lem"
{ yygotominor.yy60 = yymsp[-1].minor.yy60; }
#line 885 "parser.c"
        break;
      case 7:
#line 68 "parser.lem"
{ yygotominor.yy60 = -1*yymsp[0].minor.yy60; }
#line 890 "parser.c"
        break;
      case 8:
#line 69 "parser.lem"
{ yygotominor.yy60 = yymsp[-2].minor.yy60 - yymsp[0].minor.yy60; }
#line 895 "parser.c"
        break;
      case 9:
#line 70 "parser.lem"
{ yygotominor.yy60 = yymsp[-2].minor.yy60 + yymsp[0].minor.yy60; }
#line 900 "parser.c"
        break;
      case 10:
#line 71 "parser.lem"
{ yygotominor.yy60 = yymsp[-2].minor.yy60 * yymsp[0].minor.yy60; }
#line 905 "parser.c"
        break;
      case 11:
#line 72 "parser.lem"
{ yygotominor.yy60 = Div(yymsp[-2].minor.yy60,yymsp[0].minor.yy60,(*yymsp[-1].minor.yy0->minDiv)); }
#line 910 "parser.c"
        break;
      case 12:
#line 73 "parser.lem"
{ yygotominor.yy60 = pow(yymsp[-2].minor.yy60,yymsp[0].minor.yy60); }
#line 915 "parser.c"
        break;
      case 13:
#line 74 "parser.lem"
{ yygotominor.yy60 = fabs(yymsp[-1].minor.yy60); }
#line 920 "parser.c"
        break;
      case 14:
#line 75 "parser.lem"
{ yygotominor.yy60 = acosh(yymsp[-1].minor.yy60); }
#line 925 "parser.c"
        break;
      case 15:
#line 76 "parser.lem"
{ yygotominor.yy60 = acos(yymsp[-1].minor.yy60); }
#line 930 "parser.c"
        break;
      case 16:
#line 77 "parser.lem"
{ yygotominor.yy60 = asinh(yymsp[-1].minor.yy60); }
#line 935 "parser.c"
        break;
      case 17:
#line 78 "parser.lem"
{ yygotominor.yy60 = asin(yymsp[-1].minor.yy60); }
#line 940 "parser.c"
        break;
      case 18:
#line 79 "parser.lem"
{ yygotominor.yy60 = atanh(yymsp[-1].minor.yy60); }
#line 945 "parser.c"
        break;
      case 19:
#line 80 "parser.lem"
{ yygotominor.yy60 = atan(yymsp[-1].minor.yy60); }
#line 950 "parser.c"
        break;
      case 20:
#line 81 "parser.lem"
{ yygotominor.yy60 = cosh(yymsp[-1].minor.yy60); }
#line 955 "parser.c"
        break;
      case 21:
#line 82 "parser.lem"
{ yygotominor.yy60 = cos(yymsp[-1].minor.yy60); }
#line 960 "parser.c"
        break;
      case 22:
#line 83 "parser.lem"
{ yygotominor.yy60 = exp(yymsp[-1].minor.yy60); }
#line 965 "parser.c"
        break;
      case 23:
#line 84 "parser.lem"
{ yygotominor.yy60 = Ln(yymsp[-1].minor.yy60); }
#line 970 "parser.c"
        break;
      case 24:
#line 85 "parser.lem"
{ yygotominor.yy60 = log10(yymsp[-1].minor.yy60); }
#line 975 "parser.c"
        break;
      case 25:
#line 86 "parser.lem"
{ yygotominor.yy60 = sinh(yymsp[-1].minor.yy60); }
#line 980 "parser.c"
        break;
      case 26:
#line 87 "parser.lem"
{ yygotominor.yy60 = sin(yymsp[-1].minor.yy60); }
#line 985 "parser.c"
        break;
      case 27:
#line 88 "parser.lem"
{ yygotominor.yy60 = sqrt(yymsp[-1].minor.yy60); }
#line 990 "parser.c"
        break;
      case 28:
#line 89 "parser.lem"
{ yygotominor.yy60 = tan(yymsp[-1].minor.yy60); }
#line 995 "parser.c"
        break;
      case 29:
#line 90 "parser.lem"
{ yygotominor.yy60 = ((yymsp[-1].minor.yy60 > 0) ? yymsp[-1].minor.yy60 : 0); }
#line 1000 "parser.c"
        break;
      case 30:
#line 91 "parser.lem"
{ yygotominor.yy60 = ((yymsp[-1].minor.yy60 > 0) ? 1 : 0); }
#line 1005 "parser.c"
        break;
      case 31:
#line 93 "parser.lem"
{ yygotominor.yy60 = yymsp[0].minor.yy17.d; }
#line 1010 "parser.c"
        break;
      case 32:
#line 95 "parser.lem"
{ yygotominor.yy17.f = **yymsp[0].minor.yy0->variable; yygotominor.yy17.d = 1.0; }
#line 1015 "parser.c"
        break;
      case 33:
#line 96 "parser.lem"
{ yygotominor.yy17.f = **yymsp[0].minor.yy0->variable; yygotominor.yy17.d = 0.0; }
#line 1020 "parser.c"
        break;
      case 34:
#line 97 "parser.lem"
{ yygotominor.yy17.f = yymsp[0].minor.yy0->constant; yygotominor.yy17.d = 0.0; }
#line 1025 "parser.c"
        break;
      case 35:
#line 98 "parser.lem"
{ yygotominor.yy17.f = yymsp[-1].minor.yy17.f; yygotominor.yy17.d = yymsp[-1].minor.yy17.d; }
#line 1030 "parser.c"
        break;
      case 36:
#line 99 "parser.lem"
{ yygotominor.yy17.f = -1*yymsp[0].minor.yy17.f; yygotominor.yy17.d = -1*yymsp[0].minor.yy17.d; }
#line 1035 "parser.c"
        break;
      case 37:
#line 100 "parser.lem"
{ yygotominor.yy17.f = yymsp[-2].minor.yy17.f + yymsp[0].minor.yy17.f; yygotominor.yy17.d = yymsp[-2].minor.yy17.d + yymsp[0].minor.yy17.d; }
#line 1040 "parser.c"
        break;
      case 38:
#line 101 "parser.lem"
{ yygotominor.yy17.f = yymsp[-2].minor.yy17.f - yymsp[0].minor.yy17.f; yygotominor.yy17.d = yymsp[-2].minor.yy17.d - yymsp[0].minor.yy17.d; }
#line 1045 "parser.c"
        break;
      case 39:
#line 102 "parser.lem"
{ yygotominor.yy17.f = yymsp[-2].minor.yy17.f * yymsp[0].minor.yy17.f; yygotominor.yy17.d = yymsp[-2].minor.yy17.d*yymsp[0].minor.yy17.f + yymsp[-2].minor.yy17.f*yymsp[0].minor.yy17.d; }
#line 1050 "parser.c"
        break;
      case 40:
#line 103 "parser.lem"
{ yygotominor.yy17.f = Div(yymsp[-2].minor.yy17.f,yymsp[0].minor.yy17.f,(*yymsp[-1].minor.yy0->minDiv)); yygotominor.yy17.d = Div((yymsp[-2].minor.yy17.d*yymsp[0].minor.yy17.f - yymsp[-2].minor.yy17.f*yymsp[0].minor.yy17.d),(yymsp[0].minor.yy17.f*yymsp[0].minor.yy17.f),(*yymsp[-1].minor.yy0->minDiv)); }
#line 1055 "parser.c"
        break;
      case 41:
#line 104 "parser.lem"
{ yygotominor.yy17.f = pow(yymsp[-2].minor.yy17.f,yymsp[0].minor.yy17.f); yygotominor.yy17.d = pow(yymsp[-2].minor.yy17.f,yymsp[0].minor.yy17.f)*(yymsp[-2].minor.yy17.d*Div(yymsp[0].minor.yy17.f,yymsp[-2].minor.yy17.f,(*yymsp[-1].minor.yy0->minDiv))) + (yymsp[-2].minor.yy17.f ? (yymsp[0].minor.yy17.d*Ln(yymsp[-2].minor.yy17.f)) : 0); }
#line 1060 "parser.c"
        break;
      case 42:
#line 105 "parser.lem"
{ yygotominor.yy17.f = fabs(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(yymsp[-1].minor.yy17.f,fabs(yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1065 "parser.c"
        break;
      case 43:
#line 106 "parser.lem"
{ yygotominor.yy17.f = acosh(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = sinh(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1070 "parser.c"
        break;
      case 44:
#line 107 "parser.lem"
{ yygotominor.yy17.f = acos(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(-1,sqrt(1-yymsp[-1].minor.yy17.f*yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1075 "parser.c"
        break;
      case 45:
#line 108 "parser.lem"
{ yygotominor.yy17.f = asinh(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,sqrt(1+yymsp[-1].minor.yy17.f*yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1080 "parser.c"
        break;
      case 46:
#line 109 "parser.lem"
{ yygotominor.yy17.f = asin(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,sqrt(1-yymsp[-1].minor.yy17.f*yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1085 "parser.c"
        break;
      case 47:
#line 110 "parser.lem"
{ yygotominor.yy17.f = atanh(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,(1-yymsp[-1].minor.yy17.f*yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1090 "parser.c"
        break;
      case 48:
#line 111 "parser.lem"
{ yygotominor.yy17.f = atan(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,(1+yymsp[-1].minor.yy17.f*yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1095 "parser.c"
        break;
      case 49:
#line 112 "parser.lem"
{ yygotominor.yy17.f = cosh(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = sinh(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1100 "parser.c"
        break;
      case 50:
#line 113 "parser.lem"
{ yygotominor.yy17.f = cos(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = -1*sin(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1105 "parser.c"
        break;
      case 51:
#line 114 "parser.lem"
{ yygotominor.yy17.f = exp(yymsp[-1].minor.yy17.f);  yygotominor.yy17.d = exp(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1110 "parser.c"
        break;
      case 52:
#line 115 "parser.lem"
{ yygotominor.yy17.f = Ln(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,yymsp[-1].minor.yy17.f,(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1115 "parser.c"
        break;
      case 53:
#line 116 "parser.lem"
{ yygotominor.yy17.f = log10(yymsp[-1].minor.yy17.f); Div(1,(yymsp[-1].minor.yy17.f*Ln(10)),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1120 "parser.c"
        break;
      case 54:
#line 117 "parser.lem"
{ yygotominor.yy17.f = sinh(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = cosh(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1125 "parser.c"
        break;
      case 55:
#line 118 "parser.lem"
{ yygotominor.yy17.f = sin(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = cos(yymsp[-1].minor.yy17.f)*yymsp[-1].minor.yy17.d; }
#line 1130 "parser.c"
        break;
      case 56:
#line 119 "parser.lem"
{ yygotominor.yy17.f = sqrt(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,(2*sqrt(yymsp[-1].minor.yy17.f)),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1135 "parser.c"
        break;
      case 57:
#line 120 "parser.lem"
{ yygotominor.yy17.f = tan(yymsp[-1].minor.yy17.f); yygotominor.yy17.d = Div(1,cos(yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*Div(1,cos(yymsp[-1].minor.yy17.f),(*yymsp[-2].minor.yy0->minDiv))*yymsp[-1].minor.yy17.d; }
#line 1140 "parser.c"
        break;
      case 58:
#line 121 "parser.lem"
{ yygotominor.yy17.f = ((yymsp[-1].minor.yy17.f > 0) ? yymsp[-1].minor.yy17.f : 0); yygotominor.yy17.d = (((yymsp[-1].minor.yy17.f == 0)? 0.5 : ((yymsp[-1].minor.yy17.f > 0) ? 1.0 : 0))*yymsp[-1].minor.yy17.d); }
#line 1145 "parser.c"
        break;
      case 59:
#line 122 "parser.lem"
{ yygotominor.yy17.f = ((yymsp[-1].minor.yy17.f > 0) ? 1 : 0); yygotominor.yy17.d = ((yymsp[-1].minor.yy17.f == 0)? 1/(*yymsp[-2].minor.yy0->minDiv) : 0.0); }
#line 1150 "parser.c"
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = yyact;
      yymsp->major = yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else if( yyact == YYNSTATE + YYNRULE + 1 ){
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
#line 48 "parser.lem"
 Error("Parser is lost..."); *solution = sqrtf(-1.f);
#line 1197 "parser.c"
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  ParseARG_FETCH;
#define TOKEN (yyminor.yy0)
#line 47 "parser.lem"
 Error("Syntax Error"); *solution = sqrtf(-1.f); 
#line 1214 "parser.c"
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
#line 46 "parser.lem"
 Debug("Parsing Complete"); 
#line 1236 "parser.c"
  ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Parse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ParseTOKENTYPE yyminor       /* The value for the token */
  ParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
    /* if( yymajor==0 ) return; // not sure why this was here... */
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  ParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,yymajor);
    if( yyact<YYNSTATE ){
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      if( yyendofinput && yypParser->yyidx>=0 ){
        yymajor = 0;
      }else{
        yymajor = YYNOCODE;
      }
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else if( yyact == YY_ERROR_ACTION ){
      int yymx;
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }else{
      yy_accept(yypParser);
      yymajor = YYNOCODE;
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}
