/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include "log.h"

#include "variable.h"

variable_ * variableAttachNew(variable_ *r, char *name, char type)
{
	variable_ *var = NULL;
	
	ReturnNULLIf(r == NULL, "Not initialised");
	ReturnNULLIf(name == NULL, "Bad name");
	
	var = variableNew(var);
	ReturnNULLIf(var == NULL);
	var->name = malloc(strlen(name)+1);
	ReturnNULLIf(var->name == NULL);
	strcpy(var->name, name);
	var->type = type;
	var->next = r->next;
	r->next = var;
	
	Debug("%s", name);
	
	return var;
}

double ** variableFindData(variable_ *r, char *name, char type)
{
	variable_ *var;
	
	ReturnNULLIf(name == NULL, "Bad name");
	
	for(var = r; var != NULL; var = var->next) {
		if(var->name != NULL) {
			if(!strcmp(var->name, name) && (var->type == type)) {
				return var->data;
			}
		}
	}
	
	return NULL;
}

int variableDestroyAll(variable_ **r)
{
	variable_ *ptr, *ptrNext;
	
	ReturnErrIf(r == NULL);
	ReturnErrIf(*r == NULL);
	Debug("Destroying variables %p", *r);
	
	for(ptr = (*r); ptr != NULL; ptr = ptrNext) {
		ptrNext = ptr->next;
		if(ptr->name != NULL)
			free(ptr->name);
		free(ptr);
	}
	
	*r = NULL;
	return 0;
}

variable_ * variableNew(variable_ *r)
{
	ReturnNULLIf(r != NULL);
	r = calloc(1, sizeof(variable_));
	ReturnNULLIf(r == NULL, "Malloc Failed");
	Debug("Creating variables %p", r);
	return r;
}
