/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include <log.h>
#include "device_internal.h"

static char *type = "Current Controlled Current Source";

#define NUM_PINS	3
#define K			0
#define J			1
#define S			2

struct _devicePrivate {
	double gain;
	node_ *nodeKS;
	node_ *nodeJS;
};

static int load(device_ *r, state_ *state)
{
	devicePrivate_ *p;
	ReturnErrIf(r == NULL);
	p = r->private;
	ReturnErrIf(p == NULL);
	
	Debug("%s Load %p", type, r);
	
	/* Modified Nodal Analysis Stamp
	 *	              	+  __  -
	 *	  |_Vk_Vj_Is_|	__/  \__
	 *	k | -- --  g |	k \__/ j
	 *	j | -- -- -g |	------->
	 *	                   	Ir = g*Is
	 */
	
	ReturnErrIf(nodeDataPlus(p->nodeKS, p->gain));
	ReturnErrIf(nodeDataMinus(p->nodeJS, p->gain));
	
	return 0;
}

int deviceConfigCCCS(device_ *r, double gain)
{
	devicePrivate_ *p;
	
	ReturnErrIf(r == NULL);
	ReturnErrIf(r->private != NULL);
	ReturnErrIf(r->numPins != NUM_PINS);
	
	Debug("Configuring %s %p", type, r);
	
	r->type = type;
	r->load = load;
	
	r->private =  calloc(1, sizeof(devicePrivate_));
	ReturnErrIf(r->private == NULL);
	p = r->private;
	
	p->gain = gain;
	
	p->nodeKS = matrixFindNode(r->matrix, r->pin[K], r->pin[S]);
	ReturnErrIf(p->nodeKS == NULL);
	p->nodeJS = matrixFindNode(r->matrix, r->pin[J], r->pin[S]);
	ReturnErrIf(p->nodeJS == NULL);
	
	return 0;
}
