/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include <log.h>
#include "device_internal.h"

#define NUM_PINS	2
#define K			0
#define J			1

static char *type = "Resistor";

struct _devicePrivate {
	double G;
	node_ *nodeKK;
	node_ *nodeKJ;
	node_ *nodeJK;
	node_ *nodeJJ;
};

static int load(device_ *r, state_ *state)
{
	devicePrivate_ *p;
	ReturnErrIf(r == NULL);
	p = r->private;
	ReturnErrIf(p == NULL);
	
	Debug("%s Load %p", type, r);
	
	/* Modified Nodal Analysis Stamp
	 *	  |_Vk_Vj_|		+        -
	 *	k | G  -G |		--/\/\/\--
	 *	j |-G   G |		k        j
	 */
	
	ReturnErrIf(nodeDataPlus(p->nodeKK, p->G));
	ReturnErrIf(nodeDataMinus(p->nodeKJ, p->G));
	ReturnErrIf(nodeDataMinus(p->nodeJK, p->G));
	ReturnErrIf(nodeDataPlus(p->nodeJJ, p->G));
	
	return 0;
}

int deviceConfigResistor(device_ *r, double R)
{
	devicePrivate_ *p;
	
	ReturnErrIf(r == NULL);
	ReturnErrIf(r->private != NULL);
	ReturnErrIf(r->numPins != NUM_PINS);
	
	Debug("Configuring %s %p", type, r);
	
	r->type = type;
	r->load = load;
	
	r->private =  calloc(1, sizeof(devicePrivate_));
	ReturnErrIf(r->private == NULL);
	p = r->private;
	
	p->G = 1/R;
	
	p->nodeKK = matrixFindNode(r->matrix, r->pin[K], r->pin[K]);
	ReturnErrIf(p->nodeKK == NULL);
	p->nodeKJ = matrixFindNode(r->matrix, r->pin[K], r->pin[J]);
	ReturnErrIf(p->nodeKJ == NULL);
	p->nodeJK = matrixFindNode(r->matrix, r->pin[J], r->pin[K]);
	ReturnErrIf(p->nodeJK == NULL);
	p->nodeJJ = matrixFindNode(r->matrix, r->pin[J], r->pin[J]);
	ReturnErrIf(p->nodeJJ == NULL);
	
	return 0;
}
