/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include <log.h>
#include "device_internal.h"

static char *type = "Voltage Controlled Voltage Source";

#define NUM_PINS	4
#define K			0
#define J			1
#define P			2
#define Q			3

struct _devicePrivate {
	double gain;
	node_ *nodeRK;
	node_ *nodeRJ;
	node_ *nodeKR;
	node_ *nodeJR;
	node_ *nodeRP;
	node_ *nodeRQ;
};


static int load(device_ *r, state_ *state)
{
	devicePrivate_ *p;
	ReturnErrIf(r == NULL);
	p = r->private;
	ReturnErrIf(p == NULL);
	
	Debug("%s Load %p", type, r);
	
	/* Modified Nodal Analysis Stamp
	 *	                     	  		+  /\  -
	 *	  |_Vk_Vj_Ir_Vp_Vq_|_rhs_|		__/Vr\__
	 *	k | -- --  1 -- -- | --  |		k \  / j
	 *	j | -- -- -1 -- -- | --  |		   \/ Vr = g*Vpq
	 *	r |  1 -1 -- -g  g | --  |		------->
	 *	                    	  		   Ir
	 */
	
	ReturnErrIf(nodeDataPlus(p->nodeRK, 1.0));
	ReturnErrIf(nodeDataMinus(p->nodeRJ, 1.0));
	ReturnErrIf(nodeDataPlus(p->nodeKR, 1.0));
	ReturnErrIf(nodeDataMinus(p->nodeJR, 1.0));
	
	ReturnErrIf(nodeDataMinus(p->nodeRP, p->gain));
	ReturnErrIf(nodeDataPlus(p->nodeRQ, p->gain));
	
	return 0;
}

int deviceConfigVCVS(device_ *r, double gain)
{
	devicePrivate_ *p;
	row_ *rRow;
	
	ReturnErrIf(r == NULL);
	ReturnErrIf(r->private != NULL);
	ReturnErrIf(r->numPins != NUM_PINS);
	
	ReturnErrIf((r->pin[K] == &gndRow) && (r->pin[J] == &gndRow));
	
	Debug("Configuring %s %p", type, r);
	
	r->type = type;
	r->load = load;
	
	r->private =  calloc(1, sizeof(devicePrivate_));
	ReturnErrIf(r->private == NULL);
	p = r->private;
	
	p->gain = gain;
	
	rRow = matrixFindRow(r->matrix, r->refdes, TYPE_EXTENDED);
	ReturnErrIf(rRow == NULL);
	
	p->nodeRK = matrixFindNode(r->matrix, rRow, r->pin[K]);
	ReturnErrIf(p->nodeRK == NULL);
	p->nodeRJ = matrixFindNode(r->matrix, rRow, r->pin[J]);
	ReturnErrIf(p->nodeRJ == NULL);
	p->nodeKR = matrixFindNode(r->matrix, r->pin[K], rRow);
	ReturnErrIf(p->nodeKR == NULL);
	p->nodeJR = matrixFindNode(r->matrix, r->pin[J], rRow);
	ReturnErrIf(p->nodeJR == NULL);
	p->nodeRP = matrixFindNode(r->matrix, rRow, r->pin[P]);
	ReturnErrIf(p->nodeJR == NULL);
	p->nodeRQ = matrixFindNode(r->matrix, rRow, r->pin[Q]);
	ReturnErrIf(p->nodeJR == NULL);
	
	return 0;
}
