/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#ifndef DEVICE_INTERNAL_H
#define DEVICE_INTERNAL_H

#include "device.h"
#include "matrix.h"
#include "row.h"
#include "node.h"
#include "state.h"

typedef int (*deviceUnconfig_)(device_ *r);
typedef int (*deviceLoad_)(device_ *r, state_ *state);
typedef deviceLinear (*deviceLinearize_)(device_ *r, state_ *state);
typedef int (*deviceStep_)(device_ *r, state_ *state, double time, 
		deviceNIType niType);
typedef double (*deviceNextStep_)(device_ *r, state_ *state, 
		deviceNIType niType);
typedef double (*deviceNextBreak_)(device_ *r, state_ *state, double time);

typedef struct _devicePrivate devicePrivate_;
struct _device {
	matrix_ *matrix;
	char *refdes;
	char *type;
	row_ *pin[DEVICE_MAX_PINS];
	int numPins;
	devicePrivate_ *private;
	/* Utilities */
	deviceUnconfig_ unconfig;
	/* Operating Point */
	deviceLoad_ load;
	deviceLinearize_ linearize;
	/* Transient Analysis */
	deviceStep_ step;
	deviceNextStep_ nextStep;
	deviceNextBreak_ nextBreak;
};

#define Max2(x,y) ((x > y) ? x : y)
#define Min2(x,y) ((x > y) ? y : x)
#define Max3(x,y,z) ((x > y) ? ((z > x) ? z : x) : ((z > y) ? z : y))
#define Min3(x,y,z) ((x < y) ? ((z < x) ? z : x) : ((z < y) ? z : y))
#define DD1(xnp1, xn, hn) \
	((xnp1 - xn) / \
		(hn))
#define DD2(xnp1, xn, xnm1, hn, hnm1) \
	((DD1(xnp1, xn, hn) - DD1(xn, xnm1, hnm1)) / \
		(hn + hnm1))
#define DD3(xnp1, xn, xnm1, xnm2, hn, hnm1, hnm2) \
	((DD2(xnp1, xn, xnm1, hn, hnm1) - DD2(xn, xnm1, xnm2, hnm1, hnm2)) / \
			(hn + hnm1 + hnm2))

#endif
