/*
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness	<charles@cooper-street.com>
 */

#include <unistd.h>
#define _GNU_SOURCE
#include <getopt.h>

#include "log.h"
LogMaster;
#include "ei.h"
#include "calc.h"

#define EISPICE_MAJOR_VERSION 0
#define EISPICE_MINOR_VERSION 2

void help()
{
	Info("eispice %i.%i", EISPICE_MAJOR_VERSION, EISPICE_MINOR_VERSION);
    Info("Usage: eispice <options> <spice deck>");
	Info("Options:");
	Info("\t-v, --version : display version info");
	Info("\t-h, --help    : display help info");
	Info("\t-r, --raw <filename>   : file to save data (default is ");
	Info("\t                         rawspice.raw, '-' for stdout)");
	Info("\t-e, --error <filename> : file to print errors (default is stderr)");
	Info("\t-l, --log <filename>   : file to print log (default is stdout)");
}

void version()
{
	Info("------------------------------------------------------------------");
	Info("eispice %i.%i", EISPICE_MAJOR_VERSION, EISPICE_MINOR_VERSION);
	Info("Compiled " __DATE__ " at " __TIME__);
	Info("(c) 2006 Cooper Street Innovations Inc.");
	Info("------------------------------------------------------------------");
	eiInfo();
	Info("------------------------------------------------------------------");
	calcInfo();
	Info("------------------------------------------------------------------");
	LogInfo();
	Info("------------------------------------------------------------------");
	Info("SuperLU (Version 3.0)\n");
	Info("Copyright (c) 2003, The Regents of the University of California,");
	Info("through Lawrence Berkeley National Laboratory (subject to receipt");
	Info("of any required approvals from U.S. Dept. of Energy)");
	Info(" ");
	Info("All rights reserved.");
	Info(" ");
	Info("Redistribution and use in source and binary forms, with or without");
	Info("modification, are permitted provided that the following conditions");
	Info("are met:");
	Info(" ");
	Info("(1) Redistributions of source code must retain the above copyright");
	Info("notice, this list of conditions and the following disclaimer."); 
	Info("(2) Redistributions in binary form must reproduce the above");
	Info("copyright notice, this list of conditions and the following");
	Info("disclaimer in the documentation and/or other materials provided");
	Info("with the distribution.");
	Info("(3) Neither the name of Lawrence Berkeley National Laboratory, U.S.");
	Info("Dept. of Energy nor the names of its contributors may be used to");
	Info("endorse or promote products derived from this software without");
	Info("specific prior written permission.");
	Info(" ");
	Info("THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS");
	Info("\"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT");
	Info("LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS");
	Info("FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE");
	Info("COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,");
	Info("INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING");
	Info(",BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;");
	Info("LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER");
	Info("CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT");
	Info("LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN");
	Info("ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE");
	Info("POSSIBILITY OF SUCH DAMAGE.");
	Info("------------------------------------------------------------------");
}

int main(int argc, char *argv[])
{
	int opt;
	struct option longopts[] = { 
			{"help", 0, NULL, 'h'},
			{"version", 0, NULL, 'v'},
			{"about", 0, NULL, 'a'},
			{"raw", 1, NULL, 'r'},
			{"error", 1, NULL, 'e'},
			{"log", 1, NULL, 'l'},
			{"z", 1, NULL, 'z'},
			{0, 0, 0, 0}
    };
	FILE *rawfd = NULL;
	char *rawFile = "rawspice.raw", *inFile = "stdin";
	ei_ *ei = NULL;
	
	double d = 4.4;
	/* Process the command line options */
	while((opt = getopt_long(argc,argv,"hvar:f:e:l:z",longopts,NULL)) != -1) {
		switch(opt) {
		case 'z': bzero((char*)&d, sizeof(double));
		printf("%e", d); ExitSuccess;
		case 'a':
		case 'v': version(); ExitSuccess;
		case 'r': rawFile = optarg; break;
		case 'e': OpenErrorFile(optarg); break;
		case 'l': OpenLogFile(optarg); break;
		case 'h': help(); ExitSuccess;
		case '?': ExitFailure("Unkown option");
        case ':': ExitFailure("Option needs a value");
		default:  help(); ExitFailure("Invalid option");
		}
	}
	
	/* Open the input file, if one is defined */
	ExitFailureIf((optind < (argc - 1)), 
			"Can only process one file at a time. Try -h option for help.");
	if (optind == argc) {
		Debug("Input from stdin");
    } else if(optind == (argc - 1)) {
		Debug("Using File %s for Input", argv[argc-1]);
		inFile = argv[argc-1];
	}
	
	/* Open the output file */
	if(rawfd == NULL) {
		Debug("Openining File %s for Output", rawFile);
		if((rawFile[0] == '-') && (rawFile[1] == 0x0)) {
			rawfd = stdout;
		} else {
			rawfd = fopen(rawFile, "wb");
			ExitFailureIf(rawfd == NULL, "Failed to open file %s", rawFile);
		}
	}
	
	/* run the simulation */
	ei = eiNew(ei, inFile);
	ExitFailureIf(ei == NULL, "Failed to create spice deck");
	
	ExitFailureIf(eiProcess(ei), "Failed to process spice deck");
	ExitFailureIf(eiWriteRaw(ei, rawfd));
	
	/* close everything and exit */
	eiDestroy(&ei);
	if(rawfd != stdout) {
		Debug("Closing File %s", rawFile);
		fclose(rawfd);
	}
	CloseErrorFile;
	CloseLogFile;
	
    ExitSuccess;
}
